﻿using System;
using System.Threading;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using GHIElectronics.NETMF.FEZ;

namespace FEZDominoPressureCQ
{
    public class Program
    {
        static int interval = 60;   /* 60sec */
        static SCP1000D01 pressureDevice;
        static Weather weatherDevice;
        static Timer measurementTimer;

        public static void Main()
        {
            FEZ_Shields.KeypadLCD.Initialize();
            FEZ_Shields.KeypadLCD.TurnBacklightOn();
            FEZ_Shields.KeypadLCD.Clear();

            pressureDevice = new SCP1000D01(
                    FEZ_Pin.Digital.UEXT10, SPI.SPI_module.SPI2, FEZ_Pin.Digital.UEXT6);
            if (!pressureDevice.Initialize())
            {
                FEZ_Shields.KeypadLCD.Clear();
                FEZ_Shields.KeypadLCD.SetCursor(0, 0);
                FEZ_Shields.KeypadLCD.Print("Initialize error");
                return;
            }

            pressureDevice.Measurement();
            weatherDevice = new Weather(3600 / interval, pressureDevice.GetPressure());

            measurementTimer = new Timer(new TimerCallback(MeasurementTimer),
                    null, 0, interval * 1000);

            Thread.Sleep(Timeout.Infinite);
        }

        static void MeasurementTimer(object o)
        {
            double temperature;
            string temperatureAbsolute;
            string temperatureSign;
            double pressure;
            string pressureAbsolute;
            string pressureSign;
            double weather;
            string weatherString;

            pressureDevice.Measurement();
            pressure = pressureDevice.GetPressure();
            temperature = pressureDevice.GetTemperatureCelsius();
            weather = weatherDevice.GetWeather(pressure);

            pressure = System.Math.Round((pressure / 100) * 10) / 10;
            if (0 < pressure)
            {
                pressureAbsolute = pressure.ToString("F1");
                pressureSign = "+";
            }
            else if (pressure < 0)
            {
                pressure = -pressure;
                pressureAbsolute = pressure.ToString("F1");
                pressureSign = "-";
            }
            else
            {
                pressureAbsolute = pressure.ToString("F1");
                pressureSign = " ";
            }

            temperature = System.Math.Round(temperature * 10) / 10;
            if (0 < temperature)
            {
                temperatureAbsolute = temperature.ToString("F1");
                temperatureSign = "+";
            }
            else if (temperature < 0)
            {
                temperature = -temperature;
                temperatureAbsolute = temperature.ToString("F1");
                temperatureSign = "-";
            }
            else
            {
                temperatureAbsolute = temperature.ToString("F1");
                temperatureSign = " ";
            }

            if (50 < weather)
                weatherString = "(^-^)";
            else if (weather < -50)
                weatherString = "(T_T)";
            else
                weatherString = "(O_O)";

            FEZ_Shields.KeypadLCD.Clear();
            FEZ_Shields.KeypadLCD.SetCursor(0, 0);
            FEZ_Shields.KeypadLCD.Print(pressureSign);
            FEZ_Shields.KeypadLCD.SetCursor(0, (byte)(7 - pressureAbsolute.Length));
            FEZ_Shields.KeypadLCD.Print(pressureAbsolute);
            FEZ_Shields.KeypadLCD.SetCursor(0, 7);
            FEZ_Shields.KeypadLCD.Print("hPa");
            FEZ_Shields.KeypadLCD.SetCursor(1, 0);
            FEZ_Shields.KeypadLCD.Print(temperatureSign);
            FEZ_Shields.KeypadLCD.SetCursor(1, (byte)(7 - temperatureAbsolute.Length));
            FEZ_Shields.KeypadLCD.Print(temperatureAbsolute);
            FEZ_Shields.KeypadLCD.SetCursor(1, 7);
            FEZ_Shields.KeypadLCD.Print("\u00DFC");
            FEZ_Shields.KeypadLCD.SetCursor(0, 11);
            FEZ_Shields.KeypadLCD.Print(weatherString);
        }
    }
}
