using System;
using System.Threading;
using Microsoft.SPOT.Hardware;
using GHIElectronics.NETMF.FEZ;
using GHIElectronics.NETMF.Hardware;

namespace FEZDominoPressureCQ
{
    public class SCP1000D01
    {
        SPI spi;
        InputPort drdy;
        int pressuredata;
        int temperaturedata;

        public SCP1000D01(FEZ_Pin.Digital cspin, SPI.SPI_module spipin, FEZ_Pin.Digital drdypin)
        {
            spi = new SPI(new SPI.Configuration((Cpu.Pin)cspin,
                    false, 0, 0, false, true, 500, spipin));
            drdy = new InputPort((Cpu.Pin)drdypin, false, Port.ResistorMode.Disabled);
            pressuredata = 0;
            temperaturedata = 0;
        }

        public bool Initialize()
        {
            System.Threading.Thread.Sleep(150);

            pressureDeviceWrite8(0x06, 0x01);

            System.Threading.Thread.Sleep(60);

            int i = 0;
            while (!((pressureDeviceRead8(0x07) & 0x01) == 0))
            {
                if (6 < i)
                {
                    return false;
                }
                System.Threading.Thread.Sleep(10);
                i++;
            }

            if ((pressureDeviceRead8(0x1F) & 0x01) == 0)
                return false;

            return true;
        }

        public void Measurement()
        {
            while (true)
            {
                while (drdy.Read()) ;
                pressureDeviceWrite8(0x03, 0x0C);    /* Low power acquisition start */

                while (!drdy.Read()) ;
                temperaturedata = pressureDeviceRead16(0x21) & 0x3FFF;
                pressuredata = pressureDeviceRead8(0x1F);
                pressuredata = ((pressuredata << 16) | pressureDeviceRead16(0x20)) & 0x7FFFF;

                if ((pressureDeviceRead8(0x07) & 0x10) == 0x10)
                    pressureDeviceRead16(0x20);   /* RTERR clear */
                else
                    return;
            }
        }

        public double GetPressure()
        {
            return pressuredata / 4.0;
        }

        public double GetTemperatureCelsius()
        {
            int t = temperaturedata;
            if (8191 < t) t-= 16384;
            return t / 20.0;
        }

        private void pressureDeviceWrite8(byte addr, byte data)
        {
            byte[] write = new byte[2];
            write[0] = (byte)((addr << 2) | 0x02);
            write[1] = data;
            spi.Write(write);
        }

        private byte pressureDeviceRead8(byte addr)
        {
            byte[] write = new byte[2];
            byte[] read = new byte[2];
            write[0] = (byte)((addr << 2) | 0x00);
            write[1] = 0;
            spi.WriteRead(write, read);
            return read[1];
        }

        private ushort pressureDeviceRead16(byte addr)
        {
            byte[] write = new byte[3];
            byte[] read = new byte[3];
            write[0] = (byte)((addr << 2) | 0x00);
            write[1] = 0;
            write[2] = 0;
            spi.WriteRead(write, read);
            return (ushort)((read[1] << 8) | read[2]);
        }
    }
}